



pip check
IF %ERRORLEVEL% NEQ 0 exit /B 1
cd python/tests
IF %ERRORLEVEL% NEQ 0 exit /B 1
pytest -vv --cov pyoxigraph --cov-report term-missing:skip-covered --cov-fail-under 100
IF %ERRORLEVEL% NEQ 0 exit /B 1
exit /B 0
